
/* view.cc */

#include "view.h"
#include "prop.h" // ListField

#include "QtGui"
#include "QtWidgets"

#if QT_VERSION >= 0x040500
   #define QT_4_5
#endif

#if QT_VERSION >= 0x040600
   #define QT_4_6
#endif

/* ---------------------------------------------------------------------- */

MyView::MyView (QWidget* parent) :
   graphicsView (NULL),
   scene (NULL),
   plane (NULL),
   toolbar (NULL),
   palette (NULL),
   tree (NULL),
   prop (NULL),
   info (NULL)
{
   graphicsView = new QGraphicsView (parent);

   scene = new QGraphicsScene;
   scene->setSceneRect (0, 0, 800, 600);
   graphicsView->setScene (scene);

   // scene->addText ("Hello, world!");
   // scene->addRect (QRectF (1, 1, 100, 80), QPen (Qt::red), QBrush (Qt::lightGray));

   /* background */

   const int n = 16;
   QBitmap texture (n, n);
   texture.clear ();

   QPainter painter (& texture);
   painter.drawLine (0, 0, n-1, 0);
   painter.drawLine (0, 0, 0, n-1);

   QBrush brush (QColor ("cornflowerblue"));
   brush.setTexture (texture);
   scene->setBackgroundBrush (brush);

   /* plane */

   plane = new Area;
   plane->topLevel = true;
   plane->setPen (QPen (Qt::NoPen));
   plane->setBrush (QBrush (Qt::NoBrush));
   plane->setFlags (0);
   plane->setRect (10, 10, 630, 470);
   scene->addItem (plane);

   /* componnets */

   toolbar = new QToolBar (parent);
   palette = new MyPalette (parent);
   tree = new MyTree (parent);
   prop = new MyProp (parent);
   info = new QTextEdit (parent);
   // statusBar = new QStatusBar (parent);

   /* connections */

   tree->setView (this);

   /* toolbar */

   QAction * selectAction = new QAction (this);
   selectAction->setText ("select");
   #ifdef QT_4_6
      selectAction->setIcon (QIcon::fromTheme ("window-new"));
   #endif
   selectAction->setIcon (QIcon ("icons/arrow-135.png")); // !?
   QObject::connect (selectAction, SIGNAL(triggered()), this, SLOT(selectTool()));
   toolbar->addAction (selectAction);

   QAction * resizeAction = new QAction (this);
   resizeAction->setText ("resize");
   #ifdef QT_4_6
      resizeAction->setIcon (QIcon::fromTheme ("view-refresh"));
   #endif
   resizeAction->setIcon (QIcon ("icons/selection-resize.png")); // !?
   QObject::connect (resizeAction, SIGNAL(triggered()), this, SLOT(resizeTool()));
   toolbar->addAction (resizeAction);

   QAction * connectAction = new QAction (this);
   connectAction->setText ("connect");
   #ifdef QT_4_6
      connectAction->setIcon (QIcon::fromTheme ("go-jump"));
   #endif
   connectAction->setIcon (QIcon ("icons/leaf.png")); // !?
   QObject::connect (connectAction, SIGNAL(triggered()), this, SLOT(connectTool()));
   toolbar->addAction (connectAction);

   toolbar->addSeparator ();

   QActionGroup * group = new QActionGroup (this);
   group->addAction (selectAction);
   group->addAction (resizeAction);
   group->addAction (connectAction);
   selectAction->setCheckable (true);
   resizeAction->setCheckable (true);
   connectAction->setCheckable (true);
   selectAction->setChecked (true);

   toolbar->addWidget (palette);
}

/* ---------------------------------------------------------------------- */

void MyView::selectTool ()
{
   graphicsView->setCursor (Qt::ArrowCursor);
}

void MyView::resizeTool ()
{
   graphicsView->setCursor (Qt::SizeAllCursor);
}

void MyView::connectTool ()
{
   // graphicsView->setCursor (Qt::DragLinkCursor);
   graphicsView->setCursor (Qt::PointingHandCursor);
}

/* ---------------------------------------------------------------------- */

void MyView::addArea (Area * a)
{
   plane->addArea (a);
   #ifndef QT_4_5
      scene->addItem (a);
   #endif
}

/* ---------------------------------------------------------------------- */

void MyView::areaActivated (Area * area)
{
    if (prop != NULL)
       prop->display (area);

    if (info != NULL)
       info->append (area->getTitle () + " activated");
}

/* ---------------------------------------------------------------------- */

// #include "view.moc"

/* ---------------------------------------------------------------------- */

